/*
 ============================================================================
 Name        : rock.c
 Author      : Véronique Legrand
 Version     :
 Copyright   : Your copyright notice
 Description : see : https://projets.pasteur.fr/projects/homogeneisation-couverture-pre-assemblage/wiki
 ============================================================================
 */
#include <sys/sysctl.h>
#include <cstdlib>
#include "err.h"
#include <string.h>

#include <iostream>
#include <map>
#include <vector>
#include <unistd.h>


#include "CountMinSketch.hpp"
#include "rock_commons.h"
#include "srp.h"

#include "FqMainBackend.h"
#include "fqreader.h"
#include "fqwriter.h"
#include "Filter.hpp"
#include "main_utils.h"
#include "ROCKparams.h"


using namespace std;


// #define BENCHMARK
#ifdef BENCHMARK
#include <sys/time.h>
#include <sys/resource.h>
void printRUsage() {
    struct rusage usage;
    int res=getrusage(RUSAGE_SELF,&usage);
    std::cout<<"memory info: maxrss="<<usage.ru_maxrss<<" ixrss="<<usage.ru_ixrss<<" idrss="<<usage.ru_idrss<<" isrss="<<usage.ru_isrss<<" minflt="<<usage.ru_minflt<<" majflt="<<usage.ru_majflt<<endl;
    std::cout<<"time info: user time="<<usage.ru_utime.tv_sec<<" system time="<<usage.ru_stime.tv_sec<<endl;
}
#endif




int main(int argc,char * argv[]) {
#ifdef BENCHMARK
    cout<<"program startup"<<endl;
    printRUsage();
#endif
    srp sr;
    ROCKparams main_parms;
    main_parms.initFromMainOptsArgs(argc,argv);
    int f_id=main_parms.get_f_id();
    CMSparams parms=main_parms.getCMSparams();
    FasqQualThreshold qual_thres=main_parms.getQualThresholds();
    std::vector<IO_fq_files> single_files=main_parms.get_single_files();
    vector<PE_files> v_PE_files=main_parms.get_PE_files();

    FqBaseBackend * map_id_backend[k_max_input_files];
    Filter the_filter(parms,qual_thres);

#ifdef BENCHMARK
    cout<<"processed input args; going to start reading fastq files"<<endl;
    printRUsage();
#endif
    processInputFiles(single_files,v_PE_files,map_id_backend,main_parms.getQualThreshold(),&sr);
#ifdef BENCHMARK
    cout<<"finished loading fastq file into sr structure"<<endl;
    cout<<"size of srp structure="<<sizeof(sr)<<endl;
    printRUsage();
    cout<<"Now going to fill CMS"<<endl;
#endif

    int k=main_parms.get_k();

    the_filter.fillCMS(map_id_backend,f_id,k, &sr);

#ifdef BENCHMARK
    cout<<"finished filling CCountMinSketch"<<endl;
    // cout<<"size of CountMinSketch object="<<sizeof(*pcms)<<endl;
    printRUsage();
    cout<<"going to get approximated number of distinct k-mers in the CMS."<<endl;
#endif

    unsigned long approx_nb_k_mers=the_filter.getApproxNbDistinctKMers();
#ifdef BENCHMARK
    printRUsage();
    cout<<"found "<<approx_nb_k_mers<<" distinct k-mers"<<endl;
    cout<<"Now going to remove read with cov<kappa_prime"<<endl;
#endif

    main_parms.setFilterSize(the_filter.getSize());
    // Now, remove reads that are beneath kappa_prime
    if (parms.kappa_prime) the_filter.lowFilterCMS(map_id_backend,f_id,k,&sr);

#ifdef BENCHMARK
    cout<<"finished remove read with cov<kappa_prime"<<endl;
    // cout<<"size of CountMinSketch object="<<sizeof(*pcms)<<endl;
    printRUsage();
    cout<<"Should now write output files"<<endl;
#endif
    writeFilteredFastq(map_id_backend,f_id,sr);
#ifdef BENCHMARK
    cout<<"finished writing filtered reads"<<endl;
    printRUsage();
    cout<<"Should now free memory"<<endl;
#endif
    int i;
    for (i=1;i<=f_id;i++) {
       // cout<<"deleting backend: "<<i-1<<endl;
        delete map_id_backend[i-1];
    }

#ifdef BENCHMARK
    cout<<"going to compute collision probability"<<endl;
    printRUsage();
#endif

    float p =getCollisionProba(approx_nb_k_mers,parms.lambda);
    cout<<"estimated probability of collision:"<<p<<endl;
    cout<<"estimated number of distinct k-mers:"<<approx_nb_k_mers<<endl;

    if (main_parms.is_verbose()) main_parms.printVerboseInfo();

#ifdef BENCHMARK
    cout<<"finished,going to exit "<<endl;
    printRUsage();
#endif
	return EXIT_SUCCESS;
}
