/*
 * rock_types2.h
 *
 *  Created on: Jan 20, 2016
 *      Author: vlegrand
 *      Gather here typedefs and structures useful for the main program and "inter-modules" data exchange.
 *      I divided rock into 4 "modules":
 *      1- fqreader (parses fastq and fills 3D array of structures representing all reads).
 *      2- read-utils (utility functions for DNA reads processing)
 *      3- CMS (fill and use).
 *      4- fqwriter (writes fq filtered by CMS).
 */

#ifndef ROCK_COMMONS_H_
#define ROCK_COMMONS_H_

#include <string>
#include <vector>


/*
 * Gather here type and constants definitions that are common to some of the 4 different modules in ROCK
 * (fqreader, read processing, CMS filling, result writing) and to the main program.
 *
 */

#define k_max_input_files 15

typedef struct {
    std::string in_fq_file;
    std::string out_fq_file;
    std::string undef_fq_file;
} IO_fq_files;

typedef struct {
    IO_fq_files PE1;
    IO_fq_files PE2;
}PE_files;


typedef std::vector<unsigned long> readNumericValues;

typedef struct {
    readNumericValues single_or_PE_val;
    int idx_start_PE2; // In he case where PE reads are not treated as single;
                       // indicate at which index the single_or_PE_val array starts containing values for PE2 k-mers.
}T_read_numericValues;




#endif /* ROCK_COMMONS_H_ */
