#!/bin/sh
[ -z "$srcdir" ] && srcdir="."
echo "srcdir"=${srcdir}

# Gather here the tests that require at least 4 GB of memory to run.
## do real work
## Test high filter
echo " "
echo "##################################################################################"
echo "testing high filter"


../src/rock -C 100 -l 2 -p -i ${srcdir}/data/iofiles.args/extract_klebsiella_long_reads_100.txt -o ${srcdir}/data/iofiles.args/extract_klebsiella_long_reads_100_filtered.txt #>/dev/null || exit 15

# output files should be the same size, contain the same elements but not in the same order.
nb_PE1=`grep -c "@" ${srcdir}/data/fastq.raw/klebsiella_100_1.fq`
nb_PE2=`grep -c "@" ${srcdir}/data/fastq.raw/klebsiella_100_2.fq`


ls -l ${srcdir}/
ls -l ${srcdir}/data/
ls -l data/fastq.filtered/

[ -f "data/fastq.filtered/SRR1222430_1.filtered.fastq" ] || exit 151
[ -f "data/fastq.filtered/SRR1222430_2.filtered.fastq" ] || exit 152

nb_PE1_filtered=`grep -c "@" data/fastq.filtered/SRR1222430_1.filtered.fastq`
nb_PE2_filtered=`grep -c "@" data/fastq.filtered/SRR1222430_2.filtered.fastq`

test $nb_PE1=$nb_PE1_filtered || exit 16
test $nb_PE2=$nb_PE2_filtered || exit 17

# test low filter.
echo " "
echo "##################################################################################"
echo "testing low filter"
../src/rock -C 100 -c 99 -l 2 -p -i ${srcdir}/data/iofiles.args/extract_klebsiella_long_reads_100.txt -o ${srcdir}/data/iofiles.args/extract_klebsiella_long_reads_100_filtered.txt >/dev/null || exit 18

[ -f "data/fastq.filtered/SRR1222430_1.filtered.fastq" ] || exit 181
[ -f "data/fastq.filtered/SRR1222430_2.filtered.fastq" ] || exit 182

nb_PE1_filtered=`grep -c "@" data/fastq.filtered/SRR1222430_1.filtered.fastq`
nb_PE2_filtered=`grep -c "@" data/fastq.filtered/SRR1222430_2.filtered.fastq`

test $nb_PE1_filtered=0 || exit 19
test $nb_PE2_filtered=0 || exit 20

# test that input fastq file names can be provided in command-line.
echo " "
echo "##################################################################################"
echo "testing that input fastq file names can be provided in command line."
../src/rock -C 100 -c 1 -l 2 -p ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq ${srcdir}/data/unit/test_single.fq ${srcdir}/data/unit/test_single2.fq >/dev/null || exit 181

[ -f "klebsiella_100_1.rock.fq" ] || exit 1811
[ -f "klebsiella_100_2.rock.fq" ] || exit 1812
[ -f "test_single.rock.fq" ] || exit 1813
[ -f "test_single2.rock.fq" ] || exit 1814



# checking that output files were sorted in decreasing order of quality score. For that expect to have SRR122430.1.1 as 1rst record in filtered file.
ret=`head -4 "klebsiella_100_1.rock.fq"|grep "SRR122430.1.1"`
test $ret=2 || exit 1815

echo "erasing test result files"
rm -f "klebsiella_100_1.rock.fq" || exit 1816
rm -f "klebsiella_100_2.rock.fq" || exit 1817
rm -f "test_single.rock.fq"|| exit 1818
rm -f "test_single2.rock.fq"|| exit 1819
rm -f "klebsiella_100_1.undefined.fq" || exit 18191
rm -f "klebsiella_100_2.undefined.fq" || exit 18192
rm -f "test_single.undefined.fq" || exit 18193
rm -f "test_single2.undefined.fq" || exit 19194



# test the -v option
echo " "
echo "##################################################################################"
echo "testing verbose mode"
../src/rock -C 100 -c 1 -l 2 -p -v ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq ${srcdir}/data/unit/test_single.fq ${srcdir}/data/unit/test_single2.fq|grep "CMS size=" >/dev/null || exit 182

../src/rock -C 100 -c 1 -v -p -n 1000 ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq ${srcdir}/data/unit/test_single.fq ${srcdir}/data/unit/test_single2.fq|grep "expected probability of collision was:" >/dev/null || exit 183

echo "erasing test result files"
rm -f "klebsiella_100_1.rock.fq" || exit 1816
rm -f "klebsiella_100_2.rock.fq" || exit 1817
rm -f "test_single.rock.fq"|| exit 1818
rm -f "test_single2.rock.fq"|| exit 1819
rm -f "klebsiella_100_1.undefined.fq" || exit 18191
rm -f "klebsiella_100_2.undefined.fq" || exit 18192
rm -f "test_single.undefined.fq" || exit 18193
rm -f "test_single2.undefined.fq" || exit 19194

echo " "
echo "##################################################################################"
echo "testing rock with a quality score threshold for nucleotides."

expected_diff1="304a305,308 \
  > @SRR1222430.37 37 length=250 \
> GCCTTTTCTTTTTCCAGGGAAAACCATCCAGGAGGAACTTTATTATGGCGATGTATGAAGTCGGTACCGTCACGGGTGCCGCGTCGCAGGCACGGGTGACAGGTGCGACAACAAAATGGTCACAGGAGGCGCTGGGGATACAGCCCGGGTCGATTCTGGTGGTCTACCGCAGCGGTAGTGCTGACCTGTATGCGATCAAATCCGTGGACAGCGACACGCAACTGACGCTGACCCGGAATATCACCACCGC \
> +SRR1222430.37 37 length=250 \
> AAAABFFFFFFFGCGFGGEFFCHHEEFHHGHHAFEAGAGGGFGGFHGFHGEEEFF5BDHHBDEGCEEHGAEGGFFGFCFHCGGC@E/EEEEHFEC@BBFFFG/GFEFDDCDHHGCC<?FGDDGHFFFGFFCGGG-.AGDGBGGHHGGG?ACGCFGG99B9;?C99B0CDGGCFF;DDFFFFFFFB99:;BBF/A.:9DFFF/F?EEFFFFFF>ADAFDFFFFFFE/ADADDFFFFFA;EEFF/BFFEEFF"

expected_diff2="> @SRR1222430.37 37 length=251 \
> GCTGGCCAGCTGGTTAGCAAACGACGAGGTGCTGGCGGGTTTAGCGGGAAAAATTGCGGAAGAGGCGCCGGGAAAAGCGGGGGTGTTATTACAGGTCTACGGCAGATGCGTGTCGCTGGCCGCGGCTTTTAGCGCCTACAAGTCTGGACTTCCCCGTCGGGGGGCAACCACAATTCACCCCGGCTGTATTCCACTCGGCCCCGGTGACCCTTTTGGTGTCGCCCCCGGACACCGTGCCCGTGCCCCGGTAC \
> +SRR1222430.37 37 length=251 \
> A3>333BFA2FF4GBFFDGGCGED?FGEGGHGFFFEEG?AF13@50>///11B13@@1/>//B0?>////<//B/00??@/--:--.;:C000;:0/0009.-9:.00:-.;9/9...-;.--9@--9:////-9-9..////9/;//;9///.9-..--------..99.9.//////;-;--9-.////://9/9.;.-;-99-.//.;////-;?9;...9-9-----9;-.;.;/.-9.;/99=--;"

mkdir tmp
../src/rock -C 100 -c 1 -l 2 -p -o ${srcdir}/data/iofiles.args/output_files_noNQ_Thres.txt  ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq || exit 191

../src/rock -C 100 -c 1 -l 2 -q 2 -p -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_very_low.txt  ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq || exit 192

ret1=`diff tmp/klebsiella_100_1_very_low_qual_thres.fq tmp/klebsiella_100_1_no_qual_thres.fq`
test $ret1=0 || exit 1921
ret2=`diff tmp/klebsiella_100_2_very_low_qual_thres.fq tmp/klebsiella_100_2_no_qual_thres.fq`
test $ret2=0 || exit 1922

../src/rock -C 100 -c 1 -l 2 -q 12 -p -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_12.txt ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq||exit 193

../src/rock -C 100 -c 1 -l 2 -q 13 -p -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_13.txt ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq||exit 194

ret1=`diff tmp/klebsiella_100_1_13_qual_thres.fq tmp/klebsiella_100_1_12_qual_thres.fq|grep -c "@SRR1222430.37"`
test $ret1 = 1 || exit 1931

ret2=`diff tmp/klebsiella_100_1_13_qual_thres.fq tmp/klebsiella_100_1_12_qual_thres.fq|grep -c "length"`
test $ret2 = 2 || exit 1932

ret1=`diff tmp/klebsiella_100_2_13_qual_thres.fq tmp/klebsiella_100_2_12_qual_thres.fq|grep -c "@SRR1222430.37"`
test $ret1 = 1 || exit 1933

ret2=`diff tmp/klebsiella_100_2_13_qual_thres.fq tmp/klebsiella_100_2_12_qual_thres.fq|grep -c "length"`
test $ret2 = 2 || exit 1934

rm -fr tmp

echo " "
echo "##################################################################################"
echo "testing ROCK with a quality score threshold for nucleotides and minimum number of valid k-mer to keep a read."

mkdir tmp
../src/rock -C 100 -c 1 -l 2 -p -o ${srcdir}/data/iofiles.args/output_files_noNQ_Thres.txt  ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq || exit 191

../src/rock -C 100 -c 1 -l 2 -q 2 -m 5 -p -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_very_low.txt  ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq || exit 192

ret1=`diff tmp/klebsiella_100_1_very_low_qual_thres.fq tmp/klebsiella_100_1_no_qual_thres.fq`
test $ret1=0 || exit 19211
ret2=`diff tmp/klebsiella_100_2_very_low_qual_thres.fq tmp/klebsiella_100_2_no_qual_thres.fq`
test $ret2=0 || exit 19212


[ -f "tmp/klebsiella_100_1.undefined.fq" ] || exit 19213
[ -f "tmp/klebsiella_100_2.undefined.fq" ] || exit 19214

ret1=`cat tmp/klebsiella_100_1.undefined.fq|wc -l`
ret2=`cat tmp/klebsiella_100_2.undefined.fq|wc -l`

test $ret1=0 || exit 19215
test $ret1=0 || exit 19216

../src/rock -C 100 -c 1 -l 2 -q 13 -m 500 -p -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_13.txt ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq || exit 192161

[ -f "tmp/klebsiella_100_1.undefined.fq" ] || exit 19215
[ -f "tmp/klebsiella_100_2.undefined.fq" ] || exit 19216

ret1=`cat tmp/klebsiella_100_1.undefined.fq|wc -l`
ret2=`cat tmp/klebsiella_100_2.undefined.fq|wc -l`

test $ret1 = 400 || exit 19217
test $ret1 = 400 || exit 19218

../src/rock -C 100 -c 1 -l 2 -q 13 -m 300 -p -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_13.txt ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq || exit 192162

[ -f "tmp/klebsiella_100_1.undefined.fq" ] || exit 192191
[ -f "tmp/klebsiella_100_2.undefined.fq" ] || exit 192192

ret1=`cat tmp/klebsiella_100_1.undefined.fq|wc -l`
ret2=`cat tmp/klebsiella_100_2.undefined.fq|wc -l`

test $ret1 = 136 || exit 192193
test $ret1 = 136 || exit 192194

echo " "
echo "##################################################################################"
echo "testing ROCK with a quality score threshold for nucleotides and PE processed separately"
../src/rock -C 100 -c 1 -l 2 -q 2 -o ${srcdir}/data/iofiles.args/output_files_2_PE_separated.txt ${srcdir}/data/fastq.raw/klebsiella_5_1_bad_scores.fq,${srcdir}/data/fastq.raw/klebsiella_5_2_bad_scores.fq || exit 201

[ -f "tmp/klebsiella_5_1_bad_scores.undefined.fq" ] || exit 20100
[ -f "tmp/klebsiella_5_2_bad_scores.undefined.fq" ] || exit 20101

ret1=`cat tmp/klebsiella_5_1_bad_scores.undefined.fq|wc -l`
ret2=`cat tmp/klebsiella_5_2_bad_scores.undefined.fq|wc -l`

test $ret1=8 || exit 201100
test $ret1=8 || exit 201101

ret1=`grep -c SRR1222430.1 tmp/klebsiella_5_1_bad_scores.undefined.fq`
ret2=`grep -c SRR1222430.1 tmp/klebsiella_5_2_bad_scores.undefined.fq`

test $ret1=2 || exit 201200
test $ret1=2 || exit 201201

ret1=`grep -c SRR1222430.5 tmp/klebsiella_5_1_bad_scores.undefined.fq`
ret2=`grep -c SRR1222430.5 tmp/klebsiella_5_2_bad_scores.undefined.fq`

test $ret1=2 || exit 201300
test $ret1=2 || exit 201301

ret1=`grep -c SRR1222430.2 tmp/klebsiella_5_1_bad_scores.undefined.fq`
ret2=`grep -c SRR1222430.2 tmp/klebsiella_5_2_bad_scores.undefined.fq`

test $ret1=0 || exit 201400
test $ret1=0 || exit 201401

ret1=`grep -c SRR1222430.3 tmp/klebsiella_5_1_bad_scores.undefined.fq`
ret2=`grep -c SRR1222430.3 tmp/klebsiella_5_2_bad_scores.undefined.fq`

test $ret1=0 || exit 201500
test $ret1=0 || exit 201501

ret1=`grep -c SRR1222430.4 tmp/klebsiella_5_1_bad_scores.undefined.fq`
ret2=`grep -c SRR1222430.4 tmp/klebsiella_5_2_bad_scores.undefined.fq`

test $ret1=0 || exit 201600
test $ret1=0 || exit 201601

ret1=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
ret2=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`

test $ret1=12 || exit 201700
test $ret1=12 || exit 201701

ret1=`grep -c SRR1222430.2 tmp/klebsiella_5_1_2_qual_thres.fq`
ret2=`grep -c SRR1222430.2 tmp/klebsiella_5_2_2_qual_thres.fq`

test $ret1=2 || exit 201800
test $ret1=2 || exit 201801

ret1=`grep -c SRR1222430.3 tmp/klebsiella_5_1_2_qual_thres.fq`
ret2=`grep -c SRR1222430.3 tmp/klebsiella_5_2_2_qual_thres.fq`

test $ret1=2 || exit 201802
test $ret1=2 || exit 201803

ret1=`grep -c SRR1222430.4 tmp/klebsiella_5_1_2_qual_thres.fq`
ret2=`grep -c SRR1222430.4 tmp/klebsiella_5_2_2_qual_thres.fq`

test $ret1=2 || exit 201804
test $ret1=2 || exit 201805

rm -f tmp/klebsiella_5_1_bad_scores.undefined.fq
rm -f tmp/klebsiella_5_2_bad_scores.undefined.fq
rm -f tmp/klebsiella_5_1_2_qual_thres.fq
rm -f tmp/klebsiella_5_2_2_qual_thres.fq


echo " "
echo "##################################################################################"
echo "testing ROCK with no quality score threshold for nucleotides and PE processed separately"

../src/rock -C 100 -c 1 -l 2 -o ${srcdir}/data/iofiles.args/output_files_2_PE_separated.txt ${srcdir}/data/fastq.raw/klebsiella_5_1_bad_scores.fq,${srcdir}/data/fastq.raw/klebsiella_5_2_bad_scores.fq || exit 201

[ -f "tmp/klebsiella_5_1_bad_scores.undefined.fq" ] || exit 30100
[ -f "tmp/klebsiella_5_2_bad_scores.undefined.fq" ] || exit 30101

ret1=`cat tmp/klebsiella_5_1_bad_scores.undefined.fq|wc -l`
ret2=`cat tmp/klebsiella_5_2_bad_scores.undefined.fq|wc -l`

test $ret1=0 || exit 301100
test $ret1=0 || exit 301101



ret1=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
ret2=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`

test $ret1=20 || exit 201700
test $ret1=20 || exit 201701

ret1=`grep -c SRR1222430.2 tmp/klebsiella_5_1_2_qual_thres.fq`
ret2=`grep -c SRR1222430.2 tmp/klebsiella_5_2_2_qual_thres.fq`

test $ret1=2 || exit 301800
test $ret1=2 || exit 301801

ret1=`grep -c SRR1222430.3 tmp/klebsiella_5_1_2_qual_thres.fq`
ret2=`grep -c SRR1222430.3 tmp/klebsiella_5_2_2_qual_thres.fq`

test $ret1=2 || exit 301802
test $ret1=2 || exit 301803

ret1=`grep -c SRR1222430.4 tmp/klebsiella_5_1_2_qual_thres.fq`
ret2=`grep -c SRR1222430.4 tmp/klebsiella_5_2_2_qual_thres.fq`

test $ret1=2 || exit 301804
test $ret1=2 || exit 301805

ret1=`grep -c SRR1222430.1 tmp/klebsiella_5_1_2_qual_thres.fq`
ret2=`grep -c SRR1222430.1 tmp/klebsiella_5_2_2_qual_thres.fq`

test $ret1=2 || exit 301806
test $ret1=2 || exit 301807

ret1=`grep -c SRR1222430.5 tmp/klebsiella_5_1_2_qual_thres.fq`
ret2=`grep -c SRR1222430.5 tmp/klebsiella_5_2_2_qual_thres.fq`

test $ret1=2 || exit 301808
test $ret1=2 || exit 301809

rm -fr tmp


echo " "
echo "##################################################################################"
echo "unit testing for cms component"
../src/unit_test_cms || exit 24

     
     
